# First compile the C files which comprise the low-level networking part of the program:

echo -e "\nCompiling the C source files to produce object files. . .\n"

cd compiled_files/linux/obj/debug

gcc -c -ggdb3 -D VJC_LIB_RAW_SOCK_UNDERLYING_API=2 `wx-config --cflags` ../../../../source_code/reusable/lib_rawsock/*.c

cd - > /dev/null

echo -e "  Finished compiling the C source files.\n"


# Next compile the C++ files which comprise the greatest part of the Dynamo program:

echo -e "\nCompiling the C++ source files to produce object files. . .\n"

cd compiled_files/linux/obj/debug

g++ -c -ggdb3 -D VJC_LIB_RAW_SOCK_UNDERLYING_API=2 `wx-config --cxxflags` ../../../../source_code/*.cpp

cd - > /dev/null

echo -e "  Finished compiling the C++ source files.\n"


# Finally, link all the object files together to produce an executable file:

echo -e "\nLinking all the object files together to produce an executable file. . .\n"

cd compiled_files/linux/obj/debug

g++ -ggdb3 *.o `wx-config --libs` -lpcap -o ../../bin/debug/dynamo
echo -e "  Finished linking.\n"

cd - > /dev/null

echo -e "\nALL DONE."

# Explanation of compiler options:
# --------------------------------
#
#    -c  :  Just compile the source files to object files
#           (don't link the object files to an executable)
#
#
#    -ggdb3  :  This adds debugging symbols to the executable file
#
#
#    -D VJC_LIB_RAW_SOCK_UNDERLYING_API=2 : This defines a preproccesor
#                                           constant which indicates what
#                                           networking library to use
#                                           (2 specifies the pCap library)
#
#
#    `wx-config --cxxflags` : This is a command which gets executed and
#                             results in a list of compiler switches which
#                             are required when compiling source files that
#                             use the wxWidgets library
#
#    `wx-config --cflags`   : The same as above except used when compiling C code
#
#
# After you've compiled all the source files, you should have a load
# of object files (the filename extension is ".o").
#
# After you've got all the object files, you use the linker to link 
# them all together to create an executable file.
#
# Explanation of linker options:
#
#
#    -lpcap  :  Specifies to link with the pCap networking library
#
#
#    `wx-config --libs` : This is a command which gets executed and
#                         results in a list of linker switches which
#                         are required when linking together a program
#                         which uses the wxWidgets library
