#include "GUI_Dialog_Detailed_MAC_Info.hpp"

#include <algorithm>  /* find */

#include "parser.hpp"  /* MakeStringFromMAC, ParseIP */

#include "master_specs_for_dynamo_program.hpp" /* IntType_Qty_IP */

#include "net_types_containers.hpp"  /* MAC_Data, IP_Data */

#include "database.hpp" /* g_p_sdb */


Dialog_Detailed_MAC_Info::Dialog_Detailed_MAC_Info(wxWindow* parent,MAC_Data const *const arg_p)
 : Dialog_Detailed_MAC_Info__Auto_Base_Class( parent ), p_mac_data(arg_p)
{
    Sniffer_Database::Locker_Readonly_Accessor tmp_obj(*g_p_sdb);

    assert(this->p_mac_data != 0);

    assert(this->p_mac_data->num != 0);

    this->m_txt_MAC->ChangeValue( MakeStringFromMAC(this->p_mac_data->num) );

    this->Populate_IP_List(); /* Database has already been locked above */
}

void Dialog_Detailed_MAC_Info::Populate_IP_List()
{
    /* DATABASE MUST BE LOCKED BEFORE
       THIS FUNCTION IS CALLED! */

    typedef MAC_Data::ContainerType_IP_Data::const_iterator CI;

    for (CI p  = this->p_mac_data->ip_data_container.begin();
            p != this->p_mac_data->ip_data_container.end();
            ++p)
    {
        this->m_list_ips->Append( MakeStringFromIP(p->num) );

        /* The string returned from MakeStringFromIP is not
           destroyed until the end of the statement */
    }
}

void Dialog_Detailed_MAC_Info::OnButtonClick_Refresh( wxCommandEvent& event )
{
    this->m_list_ports->Clear();  /* Wipe out all the ports */

    wxString const original_ip_str( this->m_list_ips->GetStringSelection() );

    m_list_ips->Clear();
    m_list_ports->Clear();

    {
        Sniffer_Database::Locker_Readonly_Accessor tmp_obj(*g_p_sdb);
        this->Populate_IP_List();
    }

    this->m_list_ips->SetStringSelection(original_ip_str);

    this->OnListBox_ItemSelected_IP( *(wxCommandEvent *)0 );
        /* It's UB to create a null reference, but who cares */
}

void Dialog_Detailed_MAC_Info::OnListBox_ItemSelected_IP( wxCommandEvent& event )
{
    this->m_list_ports->Clear();  /* Wipe out all the ports */

    wxString const original_ip_str( this->m_list_ips->GetStringSelection() );

    if ( original_ip_str.empty() )
        return;

    uintIP ip;

    if ( !ParseIP(original_ip_str.char_str(),ip) )
        return;

    Sniffer_Database::Locker_Readonly_Accessor tmp_obj(*g_p_sdb);

    IP_Data const *const p_ip_data =
        std::find(p_mac_data->ip_data_container.begin(),
                  p_mac_data->ip_data_container.end(),
                                                     ip);

    if ( p_ip_data == p_mac_data->ip_data_container.end() )
        return;

    typedef IP_Data::ContainerType_Port_Data::const_iterator CIt;

    wxString str;

    for (CIt p  = p_ip_data->port_data_container.begin();
             p != p_ip_data->port_data_container.end();
             ++p)
    {
        str << p->num;

        char const *const x = Port_To_Protocol_Name(p->num);

        if ( x )
            str  << wxT(" ") << wxString( x, wxConvUTF8 );

        m_list_ports->Append(str);

        str.clear();
    }
}

void Dialog_Detailed_MAC_Info::OnButtonClick_Close( wxCommandEvent& event )
{
    this->Destroy();
}

Dialog_Detailed_MAC_Info::~Dialog_Detailed_MAC_Info()
{
    m_list_ips->Clear();    /* These calls are needed to avoid a crash  */
    m_list_ports->Clear();  /* in the destructor for wxListBox. */
}

