#include "GUI_Dialog_Main.hpp"

#include <cassert>
#include <cstddef>  /* size_t */
#include <wx/msgdlg.h>
#include <wx/string.h>

#include "reusable/lib_rawsock/rawsock.h"  /* GetListOfInterfaces */

#include "globals.hpp"  /* currently_open_network_interface */
#include "database.hpp" /* g_p_sdb */

#include "GUI_Dialog_Detailed_MAC_Info.hpp"
#include "GUI_Dialog_Show_All_IPs.hpp"
#include "GUI_Dialog_About.hpp"

#include "parser.hpp"  /* MakeStringFromIP */

Dialog_Main *g_p_dlgmain = 0;

Dialog_Main::Dialog_Main( wxWindow* parent )
  : Dialog_Main__Auto_Base_Class( parent ),
    timer_refresh_currently_arping(this)
{

    this->Connect( wxEVT_TIMER, wxTimerEventHandler( Dialog_Main::OnTimerRefreshCurrentlyArping ) );

    this->Connect( g_event_type_sniffer, EventHandler_Sniffer(Dialog_Main::OnSnifferCommunicate) );

    this->p_sniffer = 0;
    this->p_arper = 0;
    //this->p_prober = 0; /* FIX */

    this->last_open_tab = 1; /* Default = 1 = Arper Tab */

    bool does_combo_box_contain_item = false;

    for (std::size_t i = 0; ; ++i)
    {
        char const *p = GetListOfInterfaces(i);

        if ( !p )
            break;

        if ( !*p )
            continue;

        this->m_combo_NetworkInterface->Append( wxString(p,wxConvUTF8) );

        does_combo_box_contain_item = true;
    }

    if (does_combo_box_contain_item)
        this->m_combo_NetworkInterface->SetSelection(0);

    /* ------ Add an item to get the column sizes, then delete it ------ */
    this->m_listctrl_data->InsertColumn(0, wxT("MAC Address"));
	this->m_listctrl_data->InsertColumn(1, wxT("Prv"));
    this->m_listctrl_data->InsertColumn(2, wxT("Pub"));
    this->m_listctrl_data->InsertColumn(3, wxT("IP Addresses"));
    this->m_listctrl_data->InsertColumn(4, wxT("Ports"));

    this->m_listctrl_data->InsertItem(0,wxT("FF:FF:FF:FF:FF:FFZZZ"));
    this->m_listctrl_data->SetItem(0,1,wxT("999ZZZ"));
    this->m_listctrl_data->SetItem(0,2,wxT("999ZZZ"));
    this->m_listctrl_data->SetItem(0,3,wxT("999.999.999.999 999.999.999.999ZZZ"));
    this->m_listctrl_data->SetItem(0,4,wxT("999 999 999 999ZZZZ"));

    this->m_listctrl_data->SetColumnWidth(0,wxLIST_AUTOSIZE);
    this->m_listctrl_data->SetColumnWidth(1,wxLIST_AUTOSIZE);
    this->m_listctrl_data->SetColumnWidth(2,wxLIST_AUTOSIZE);
    this->m_listctrl_data->SetColumnWidth(3,wxLIST_AUTOSIZE);
    this->m_listctrl_data->SetColumnWidth(4,wxLIST_AUTOSIZE);

    this->m_listctrl_data->DeleteItem(0);
    /* ------ Finished playing with column sizes ------ */


    this->m_combo_NetworkInterface->SetFocus();
    this->m_button_Toggle_Open_Interface->SetDefault();

    this->SetTitle( this->GetTitle() += wxT(

#ifdef WIN32
    "msw"
#else
    "linux"
#endif

    ")" ) );
}

Dialog_Main::~Dialog_Main()
{
    if ( this->p_sniffer ) /* If this is true then they clicked CLOSE */
    {
        this->Code_Toggle_Open_Interface();
    }

    /*
        Consider closing the open socket (if any)
        and ending all the threads (if any)
    */


    /* Consider disconnecting the SnifferCommunicate event */

}

void Dialog_Main::OnClose( wxCloseEvent& event )
{
    this->Destroy();
}

void Dialog_Main::OnButtonClick_Quit( wxCommandEvent& event )
{
    this->Destroy();
}

void Dialog_Main::OnButtonClick_About( wxCommandEvent &event )
{
/*
    wxMozillaLikeAboutBoxDialog *p;

    NEW_NULL_FAIL(p, wxMozillaLikeAboutBoxDialog(this) );

    if (!p)
        return;

    p->Show();
*/

/*
    wxAboutDialogInfo info;
    info.SetName( wxT("Dynamo") );
    info.SetVersion( wxT("0") );
    info.SetDescription( wxT("Network analysis and hacking tool") );
    info.SetWebSite( wxT("virjacode.com"), wxT("Virjacode") );


    wxAboutBox(info);
*/

    try
    {
        (new Dialog_About(this))->Show();
    }
    catch (...) {}
}

void Dialog_Main::OnSnifferCommunicate(EventClass_Sniffer &evt)
{
    /* DANGEROUS: The MAC Database may have been wiped
       by the time we get here. */

    if ( IS_BAD_ETHERSOCK(g_s) )
    {
        /* If we reach here, it means that the
           network interface was closed and
           that the MAC Database was wiped.

           This current event is a lingering
           one that arrived late after the
           network interface was closed. This
           event should be ignored. */

           return;
    }

    this->Add_to_listbox(evt);
}

void Dialog_Main::OnNotebook1_Page_Changed( wxNotebookEvent& event )
{
    /* All this function does is set the
       default button whenever a new tab
       is selected.
    */

    switch ( event.GetSelection() )
    {
    case 0: /* Step 1: Open Network Interface */

        m_button_Toggle_Open_Interface->SetDefault();
        m_combo_NetworkInterface->SetFocus();
        break;

    case 1: /* Step 2: Send ARP Requests */

        m_arptab_button_Toggle_Send_ARPs->SetDefault();
        m_arptab_txt_networks->SetFocus();
        break;

#if 0
    case 2: /* Step 3: Send Internet Probes */

        m_probetab_button_Toggle_Send_Probes->SetDefault();
        m_probetab_txt_mac_src->SetFocus();
        break;
#endif

    default:

        assert("What the fuck's going on?!" == 0);
    }

    event.Skip();  /* Needed for MS-Windows */
}

void Dialog_Main::OnNotebook1_Page_Changing( wxNotebookEvent& event )
{
    if ( IS_BAD_ETHERSOCK(g_s) ) /* True if there's no raw socket open */
    {
        /* It should not be possible for m_notebook1->GetSelection != 0
           because you can't change to any other tab unless there's a
           raw socket already open. */

        assert( event.GetOldSelection() == 0 );

        event.Veto();

        wxMessageBox(wxT("Step 1:\n\n"
                         "You must open a network\n"
                         "interface before proceeding"),
                     wxT("DANGER WILL ROBINSON"),
                     wxOK|wxICON_ERROR,
                     this);
    }
    else if (this->p_arper)
    {
        Code_Toggle_Send_ARPs();  /* Automatically stops the Arper */
    }
    else if (this->p_sniffer)
    {
        Code_Toggle_Send_Probes(); /* Automatically stops the Prober */
    }
}

void Dialog_Main::OnButtonClick_Toggle_Open_Interface(wxCommandEvent &event)
{
    this->Code_Toggle_Open_Interface();
}

void Dialog_Main::OnButtonClick_Toggle_Send_ARPs(wxCommandEvent &event)
{
    this->Code_Toggle_Send_ARPs();
}

void Dialog_Main::OnButtonClick_Toggle_Send_Probes(wxCommandEvent &event)
{
    Code_Toggle_Send_Probes();
}


void Dialog_Main::OnTimerRefreshCurrentlyArping( wxTimerEvent &event )
{
    if ( !this->p_arper )
    {
        this->m_arptab_static_status->SetLabel(wxT("Status: Not doing anything"));
        this->m_arptab_static_status->GetParent()->Layout();
        return;
    }

    uintIP const ip = this->p_arper->current_ip.Get();

    if ( !ip )
    {
        this->m_arptab_static_status->SetLabel(wxT("Status: Not doing anything"));
        this->m_arptab_static_status->GetParent()->Layout();
        return;
    }

    wxString str(wxT("Arping: "));

    str << MakeStringFromIP( ip );

    this->m_arptab_static_status->SetLabel(str);

    this->m_arptab_static_status->GetParent()->Layout();
    //This causes flickering on Windows

/*
    static bool monkey = true;

    Dialog_Main::SetTitle(monkey ? wxT("Dog") : wxT("Cat"));

    monkey = !monkey;
*/

/*
    for (MAC_Data *p = g_mac_db.m_macs; p->num; ++p)
    {
        if (-1 == m_listctrl_data->FindItem(-1,MakeStringFromMAC(p->num))  )
            m_listctrl_data->InsertItem(0,MakeStringFromMAC(p->num));
    }
*/

    //event.Skip();
}

void Dialog_Main::OnButtonClick_See_Detailed_MAC_Info( wxCommandEvent& event )
{
    int const int_count = this->m_listctrl_data->GetItemCount();

    if (int_count < 0)
        return;

    unsigned const count = int_count;

    for (unsigned i = 0; i != count; ++i)
    {
        if ( m_listctrl_data->GetItemState(i,wxLIST_STATE_SELECTED) )
        {
            /* We reach here if we find the first selected item */

            MAC_Data const *p;

            {
                Sniffer_Database::Locker_Readonly_Accessor locker(*g_p_sdb);

                if ( i >= locker.Get_MAC_Container().size() )
                    return;

                p = &locker.Get_MAC_Container()[i];

                /* Make sure mutex is unlocked before
                   dialog box is shown with next command */
            }

            (new Dialog_Detailed_MAC_Info(this,p))->ShowModal();
                /* This function call doesn't return until the
                   dialog disappears, however wxWidgets has some
                   sort of longjmp magic going on that lets it
                   jump out and process other messages, then come
                   back to finish this one. */

            return;
        }
    }

    wxMessageBox(wxT("It would help a lot if you\n"
                     "actually selected a MAC address\n"
                     "before clicking this button"),
                     wxT("DANGER WILL ROBINSON"),
                     wxOK|wxICON_ERROR,
                     this
                );
}

void Dialog_Main::OnButtonClick_See_All_IPs( wxCommandEvent& event )
{
    try
    {
        Dialog_Show_All_IPs *p = new Dialog_Show_All_IPs(this);

        p->Show();
    }
    catch (...) {}
}

void Dialog_Main::OnButtonClick_Select_All( wxCommandEvent& event )
{
    int const count = m_listctrl_data->GetItemCount();

    for (int i = 0; i != count; ++i)
    {
        m_listctrl_data->SetItemState(i,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
    }
}

void Dialog_Main::OnButtonClick_De_Select_All( wxCommandEvent& event )
{
    int const count = m_listctrl_data->GetItemCount();

    for (int i = 0; i != count; ++i)
    {
        m_listctrl_data->SetItemState(i,0,wxLIST_STATE_SELECTED);
    }
}

void Dialog_Main::OnButtonClick_Invert_Selection( wxCommandEvent& event )
{
    int const count = m_listctrl_data->GetItemCount();

    for (int i = 0; i != count; ++i)
    {
        long state = m_listctrl_data->GetItemState(i,wxLIST_STATE_SELECTED);

        if (state & wxLIST_STATE_SELECTED)
        {
            state = 0;
        }
        else
        {
            state = wxLIST_STATE_SELECTED;
        }

        m_listctrl_data->SetItemState(i,state,wxLIST_STATE_SELECTED);
    }
}

#if 0
=================================================================================
    if ( currently_network_interface_open )  /* True if sniffer is running */
    {
        if ( wxOK == wxMessageBox(wxT("The network interface must be closed\n"
                                      "before you can view detailed information\n"
                                      "for a particular MAC address.\n\n"

                                      "Do you wish to close the network interface\n"
                                      "and proceed to view the MAC address info?"),
                                      wxT("DANGER WILL ROBINSON"),
                                      wxOK|wxCANCEL|wxICON_QUESTION,
                                      this
                                     )
                                  )
        {
            Code_Toggle_Open_Interface();
        }
        else
        {
            /* Looks like the user doesn't wanna close the network interface! */
            return;
        }
    }
 #endif
