#include "GUI_Dialog_Main.hpp"

#include "parser.hpp" /* MakeStringFromMAC, MakeStringFromIP */
#include "ip_classify.hpp"  /* QuantityIPs */
#include "net_types_basic.hpp" /* uintMac, uintIP, uintPort */

void Dialog_Main::Add_to_listbox(EventClass_Sniffer &evt)
{
    if ( !evt.osdfm.mac )
        return;

    long i = this->m_listctrl_data->FindItem( -1, MakeStringFromMAC(evt.osdfm.mac) );

    if (-1 == i)  /* if MAC address isn't present in list box */
    {
        i = this->m_listctrl_data->GetItemCount();

        this->m_listctrl_data->InsertItem( i, MakeStringFromMAC(evt.osdfm.mac) );
    }

    m_listctrl_data->SetItem(i,1, wxString() << evt.osdfm.qty_private_ip); /* Private IP's Column */
    m_listctrl_data->SetItem(i,2, wxString() << evt.osdfm.qty_public_ip); /* Public IP's Column */

    if (evt.osdfm.qty_public_ip)
    {
        m_listctrl_data->SetItemTextColour(i,*wxWHITE);
        m_listctrl_data->SetItemBackgroundColour(i,*wxBLACK);
    }

    wxString str;

    for ( On_Screen_Data_For_MAC::CI_IP cip = evt.osdfm.container_of_ips.begin();
                                        cip != evt.osdfm.container_of_ips.end();
                                        ++cip)
    {
        str += MakeStringFromIP( *cip );
        str += wxT("  ");
    }

    m_listctrl_data->SetItem(i,3,str);

    /* ---- Now the ports ---- */

    str.clear();

    for ( On_Screen_Data_For_MAC::CI_Port cport = evt.osdfm.container_of_ports.begin();
                          cport != evt.osdfm.container_of_ports.end();
                          ++cport)
    {
        str += (wxString() << *cport);
        str += wxT("  ");
    }

    m_listctrl_data->SetItem(i,4,str);
}


#if 0
void Dialog_Main::Update_IP_List(MAC_Data const *const p_mac_data)
{
    /* Currently, this p_mac_data doesn't
       belong to the main global database,
       so there's no need for Critical Section
    */

    assert(p_mac_data != 0);

    long const i = m_listctrl_data->FindItem( -1, MakeStringFromMAC(p_mac_data->num) );

    if (-1 == i)  /* MAC address isn't present in list box */
        return;

    typedef MAC_Data::Container_Type_IP::const_iterator CI;

    CI p_ip_data = p_mac_data->ip_data_container.begin();

    wxString str;

    for (uint_fast8_t i = 0; i != const_max_qty_ips_on_screen; ++i, ++p_ip_data)
    {
        if ( p_ip_data == p_mac_data->ip_data_container.end() )
            break;

        str << MakeStringFromIP(p_ip_data->num) << wxT(' ');
    }

    m_listctrl_data->SetItem(i,3,str);  /* IP Column */

    str.Empty();
    str << IP_Classifier::QuantityIPs(p_mac_data,false);
    m_listctrl_data->SetItem( i, 1, str ); /* Private IP's Column */

    str.Empty();
    IP_Qty const qty = IP_Classifier::QuantityIPs(p_mac_data,true);
    str << qty;
    m_listctrl_data->SetItem( i, 2, str ); /* Public IP's */

    if (qty)
    {
        m_listctrl_data->SetItemTextColour(i,*wxWHITE);
        m_listctrl_data->SetItemBackgroundColour(i,*wxBLACK);
    }
}

void Dialog_Main::Add_Port_to_list(MAC_Data const *const p_mac_data,uintIP ip,uintPort port)
{
    /* Currently, this p_mac data doesn't
       belong to the main global database,
       so there's no need for Critical Section
    */

    long const i = m_listctrl_data->FindItem( -1, MakeStringFromMAC(p_mac_data->num) );

    if (-1 == i)
        return;

    wxString str;

    typedef MAC_Data::Tiny_Container_Type_Port::const_iterator CI;

    for (CI p_port_data  = p_mac_data->tiny_container_for_screen_ports.begin();
            p_port_data != p_mac_data->tiny_container_for_screen_ports.end();
            ++p_port_data)
    {
        str << p_port_data->num << wxT("  ");
    }

    m_listctrl_data->SetItem(i,4,str);  /* Port Column */
}
#endif
