#include "GUI_Dialog_Main.hpp"

#include <wx/thread.h>  /* wxThread */
#include <wx/msgdlg.h>  /* wxMessageBox */

#include "globals.hpp"  /* g_s, Is_There_A_Network_Interface_Currently_Open */

#include "database.hpp" /* g_sdb */

#include "reusable/lib_rawsock/rawsock.h"  /* OpenRawEtherSock */

#include "parser.hpp"   /* GetNetworksFromString, MakeStringFromMAC */

#include "sniffer.hpp"  /* For the Sniffer Thread! */
#include "arper.hpp"    /* For the Arper Thread! */

void Dialog_Main::Code_Toggle_Open_Interface(void)
{
    if ( this->p_sniffer ) /* If this is true then they clicked CLOSE */
    {
        /* ============= They clicked CLOSE =============== */

        this->last_open_tab = this->m_notebook1->GetSelection();

        assert(this->last_open_tab != -1);  /* There must have been a tab selected! */

        if ( 0 == this->last_open_tab )
            this->last_open_tab = 1;
              /* If previous active tab was "Open Interface", make it go to Arper Tab */

        this->m_notebook1->SetSelection(0);
            /* Setting the Tab control to the first tab automatically
               disables the arper thread or the prober thread
               because we handle the PAGE_CHANGING event */

        this->Toggle_Grey_OpenSocket(false);

        this->m_button_Toggle_Open_Interface->SetLabel( wxT("Open") );
        this->m_button_Toggle_Open_Interface->GetParent()->Layout();

        this->m_sockettab_static_advice->SetLabel( wxT("\n\nType the name of your network interface\ninto the text box above and hit Open.") );
        this->m_sockettab_static_advice->GetParent()->Layout();

        this->m_combo_NetworkInterface->SetFocus();

        /* Must stop the sniffer before closing the socket! */

        //SnifferThread::Stop_Waiter(*p_sniffer).Set_As_Waiting();

        this->p_sniffer->is_waiting.Set(true);

        //wxMessageBox(wxT("About to call Wait on the Sniffer Thread"));

        {
            wxWindowDisabler locker;
            this->p_sniffer->Wait();  /* Doesn't return until thread terminates */
        }

        //wxMessageBox(wxT("Finished waiting on the Sniffer Thread"));

        delete this->p_sniffer;

        this->p_sniffer = 0;

        Close_Already_Open_Raw_Socket_And_Invalidate_Handle();

        g_source_mac_for_open_iface = 0;

        //this->m_listctrl_data->DeleteAllItems();  Leave the listbox alone

        //g_p_sdb->Reset_Database(); No mutex needed because no other thread is accessing the database.

        /* --- What to do, there might already be events in
               the event queue which tell the wxListCtrl to
               add an item! --- */

        return;
    }

    /* ==================================================================== */
    /* ---- They clicked OPEN ---- */

    wxString ifacename = this->m_combo_NetworkInterface->GetValue();

    g_s = OpenRawEtherSock( ifacename.char_str(), true );

    if ( IS_BAD_ETHERSOCK(g_s) )
    {
        g_s = OpenRawEtherSock( ifacename.char_str(), false );

        if ( !IS_BAD_ETHERSOCK(g_s) )
        {
            wxMessageBox(wxT("Your network interface could not be opened\n"
                             "in Promiscuous Mode.\n\n"
                             "This might be a limitation of the actual\n"
                             "hardware you're using, or more likely it is\n"
                             "a limitation of the specific driver that's\n"
                             "controlling your hardware.\n\n"
                             "Promiscuous Mode means that the interface\n"
                             "will take in a frame even if the frame's\n"
                             "destination MAC address is not equal to\n"
                             "the interface's own MAC address.\n\n"
                             "Your interface has been opened in\n"
                             "non-promiscuous mode. The \"Send ARP Requests\"\n"
                             "feature probably won't work for you."
                             ),

                          wxT("Cannot open in Promiscuous Mode"),

                          wxOK|wxICON_ERROR,

                          this

                          );
        }
    }

    if ( IS_BAD_ETHERSOCK(g_s) )
    {
        /* ------ The socket failed to open ------ */

        this->m_sockettab_static_advice->SetLabel( wxT("\nERROR: Could not open the specified network interface.\n"
                                                 "Maybe you typed the name wrong, or maybe you don't\n"
                                                 "have the priviledges required to open a raw socket?") );

        this->m_sockettab_static_advice->GetParent()->Layout();

        this->m_combo_NetworkInterface->SetFocus();

        return;
    }

    /* ---- The socket was opened successfully ---- */

    this->m_listctrl_data->DeleteAllItems();

    g_p_sdb->Reset_Database();

    g_source_mac_for_open_iface = GetInterfaceMAC( ifacename.char_str() );

    if ( !g_source_mac_for_open_iface )
    {
        g_source_mac_for_open_iface = 0x0102030405llu;

        wxMessageBox(wxT("  ERROR: Could not determine MAC address\n"
                          "of the interface you selected.\n\n"
                          "  The following MAC address will be used\n"
                          "as the source address when sending frames:\n\n"
                          "  00:01:02:03:04:05"
                         ),

                      wxT("MAC not known"),

                      wxOK|wxICON_ERROR,

                      this

                      );
    }

    /* ---------- Now we TRY to start the sniffer thread! ---------- */

    try
    {
        this->p_sniffer = 0;

        this->p_sniffer = new SnifferThread();

        if ( this->p_sniffer->Create() != wxTHREAD_NO_ERROR )
            throw -1;

        this->p_sniffer->SetPriority(75);

        if ( this->p_sniffer->Run() != wxTHREAD_NO_ERROR )
            throw -1;
    }
    catch (...)
    {
        delete this->p_sniffer;

        this->p_sniffer = 0;

        wxMessageBox(wxT("ERROR: Unable to spawn Sniffer thread!"),wxT("DANGER WILL ROBINSON"),wxOK|wxICON_ERROR,this);
        return;
    }

    /* OK, the Sniffer thread has been started */

    this->Toggle_Grey_OpenSocket(true);

    this->m_button_Toggle_Open_Interface->SetLabel( wxT("CLOSE") );
    this->m_button_Toggle_Open_Interface->GetParent()->Layout();

    this->m_sockettab_static_advice->SetLabel( wxT("\nYour network interface is already open!\n\n"
                                             "You may proceed to Step 2!") );

    this->m_sockettab_static_advice->GetParent()->Layout();

    this->m_notebook1->SetSelection(last_open_tab);

}

void Dialog_Main::Code_Toggle_Send_ARPs(void)
{
    /* First we need to see if they clicked "Begin" or "Stop" */

    if ( this->p_arper )  /* True if Arper thread was running */
    {
        /* ------ They clicked STOP ------ */
        this->p_arper->current_ip.Set(0);

        this->timer_refresh_currently_arping.Stop();

        Toggle_Grey_SendARP(false);

        m_arptab_static_status->SetLabel(wxT("Status: Not doing anything"));
        m_arptab_button_Toggle_Send_ARPs->SetLabel(wxT("Go!"));
        m_arptab_button_Toggle_Send_ARPs->GetParent()->Layout();


        this->p_arper->is_waiting.Set(true);

        //wxMessageBox(wxT("About to call Wait on the Arper Thread"));

        {
            wxWindowDisabler locker;
            this->p_arper->Wait();  /* Doesn't return until thread terminates */
        }

        //wxMessageBox(wxT("Finished waiting on the Arper Thread"));

        delete this->p_arper;

        this->p_arper = 0;

        return;
    }

    /* ---------- Alright so they clicked start! ----------*/

    uint_fast16_t const a = this->m_arptab_spin_ms_btwn_arps->GetValue();

    uint_fast8_t const b = this->m_arptab_spin_send_qty_each_time->GetValue();

    if (a == 0 || a > 65535u)
        return;

    if (b == 0 || b > 255u)  /* Don't continue if Qty Arps is bad*/
        return;


    /* ---------- Now we TRY to start the Arper thread! ---------- */

    try
    {
        this->p_arper = 0;

        this->p_arper = new ArperThread(a,b);

        if ( !GetNetworksFromString(m_arptab_txt_networks->GetValue(),this->p_arper->nets) )
        {
            wxMessageBox(wxT("You made a mistake in the list\n"
                             "of network addresses.\n\n"

                             "Take a close look at it, correct\n"
                             "the mistake and try again!"),
                         wxT("DANGER WILL ROBINSON"),
                         wxOK|wxICON_ERROR,
                         this
                         );
            return;

            throw -1;
        }


        if ( this->p_arper->Create() != wxTHREAD_NO_ERROR )
            throw -1;

        this->p_arper->SetPriority(10);

        if ( this->p_arper->Run() != wxTHREAD_NO_ERROR )
            throw -1;
    }
    catch (...)
    {
        delete this->p_arper;

        this->p_arper = 0;

        wxMessageBox(wxT("ERROR: Unable to spawn Arper thread!"),wxT("DANGER WILL ROBINSON"),wxOK|wxICON_ERROR,this);
        return;
    }

    Toggle_Grey_SendARP(true);

    //m_arptab_static_status->SetLabel( wxT("Status: Currently Sending ARP Requests!") );
    m_arptab_button_Toggle_Send_ARPs->SetLabel( wxT("STOP") );
    m_arptab_button_Toggle_Send_ARPs->GetParent()->Layout();

    this->timer_refresh_currently_arping.Start(200);
}

void Dialog_Main::Code_Toggle_Send_Probes(void)
{
#if 0
    /* First we need to see if they clicked "Begin" or "Stop" */

    if ( this->p_prober )
    {
        this->Toggle_Grey_SendProbes(false);
        this->m_probetab_static_status->SetLabel(wxT("Status: Not doing anything"));
        this->m_probetab_button_Toggle_Send_Probes->SetLabel(wxT("Go!"));
        this->m_probetab_button_Toggle_Send_Probes->GetParent()->Layout();

        p_prober = 0;

        return;
    }

    Toggle_Grey_SendProbes(true);

    m_probetab_static_status->SetLabel(wxT("Status: Currently Sending Internet Probes!"));
    m_probetab_button_Toggle_Send_Probes->SetLabel(wxT("STOP"));
    m_probetab_button_Toggle_Send_Probes->GetParent()->Layout();

    p_prober = (ProberThread*)(1);
#endif
}
