#include "GUI_Dialog_Show_All_IPs.hpp"

#include "master_specs_for_dynamo_program.hpp"  /* IntType_Qty_IP_Ever_Logged */

#include "parser.hpp" /* MakeStringFromIP */

#include "net_types_containers.hpp"  /* MAC_Data, IP_Data */

#include "database.hpp"  /* g_p_sdb */

#include <algorithm>  /* std::find */

#include <vector>

typedef std::vector<uintIP> Tmp_IP_Container_Type;

static bool Attempt_Unique_Addition_To_Tmp_IP_Container(Tmp_IP_Container_Type &container,
                                                        uintIP const val)
{

    /* No need for a critical section in here */

    if ( container.end() != std::find( container.begin(), container.end(), val) )
        return false;  /* If it already exists, just return false */

    try
    {
        container.push_back(val);
    }
    catch (...)
    {
        return false;
    }

    return true;
}

void Dialog_Show_All_IPs::PopulateLists(void)
{
    Tmp_IP_Container_Type tmp_cont;

    { /* Begin code block for locker */

        Sniffer_Database::Locker_Readonly_Accessor locker(*g_p_sdb);

        try
        {
            tmp_cont.reserve( locker.GetQty_IP_Ever_Logged() );
        }
        catch (...)
        {
            wxMessageBox(wxT("Insufficient free memory available\n"
                             "to populate IP lists.\n\n"
                             "Free up some memory and try again."),
                         wxT("DANGER WILL ROBINSON"),
                         wxOK|wxICON_ERROR,
                         this);

             return;
        }

        typedef Sniffer_Database::ContainerType_MAC::const_iterator CI_MAC_Data;

        for ( CI_MAC_Data p_mac_data = locker.Get_MAC_Container().begin();
                        p_mac_data  != locker.Get_MAC_Container().end();
                                                            ++p_mac_data)
         {

            typedef MAC_Data::ContainerType_IP_Data::const_iterator CI_IP_Data;

            for (CI_IP_Data p_ip_data  = p_mac_data->ip_data_container.begin();
                            p_ip_data != p_mac_data->ip_data_container.end();
                                                                  ++p_ip_data)
            {
                Attempt_Unique_Addition_To_Tmp_IP_Container(tmp_cont, p_ip_data->num);
            }
        }

    } /* End code block for locker */

    std::sort( tmp_cont.begin(), tmp_cont.end() );

    for ( Tmp_IP_Container_Type::const_iterator p  = tmp_cont.begin();
                                                p != tmp_cont.end();
                                                              ++p )
    {
        if ( IP_Classifier::IsPublicIP(*p) )
            m_list_public_ips->Append( MakeStringFromIP(*p) );
        else
            m_list_private_ips->Append( MakeStringFromIP(*p) );

        /* The string returned from MakeStringFromIP is not destroyed until the end of the statement */
    }
}

Dialog_Show_All_IPs::Dialog_Show_All_IPs( wxWindow* parent )
 : Dialog_Show_All_IPs__Auto_Base_Class( parent )
{
    this->PopulateLists();
}

void Dialog_Show_All_IPs::OnButtonClick_Refresh( wxCommandEvent& event )
{
    wxString const str_prv( this->m_list_private_ips->GetStringSelection() ),
                   str_pub( this->m_list_public_ips->GetStringSelection() );

    m_list_private_ips->Clear();
    m_list_public_ips->Clear();

    this->PopulateLists();

    this->m_list_private_ips->SetStringSelection(str_prv);

    this->m_list_public_ips->SetStringSelection(str_pub);
}

void Dialog_Show_All_IPs::OnButtonClick_Close( wxCommandEvent& event )
{
    this->Destroy();
}
