#include "arper.hpp"

#include <cassert>
#include <stdint.h>

#include "globals.hpp" /* g_s (The raw socket handle) */

#include "parser.hpp"  /* MAC and IP to make literals */

#include "reusable/lib_rawsock/send.h"  /* SendARP */

bool ArperThread::DoOneNetwork(NetworkAddress const network_to_arp)
{
    uintIP max = ~(long unsigned)(const_ip_max << (32 - network_to_arp.qty_net_bits)) & const_ip_max;
                   /* Need to AND it in case we have big ints on some systems */

    max |= network_to_arp.addr;

    for (uintIP ip = network_to_arp.addr;;)
    {
        ++ip;   /* e.g. Go from   192.168.0.0   to   192.168.0.1 */

        if (ip == max)
        {
            /* Stop if we reach the "broadcast address", e.g. 192.168.0.255 */
            break;
        }

#if 0
        if (ip == g_src_ip)
        {
            /* Don't send an ARP request for the "phantom" machine */

            continue;
        }
#endif

        for (uint_fast8_t i = 0; i != this->qty_arp_each_address; ++i)
        {
            SendARP(g_s,
                    /* MAC dest */  const_mac_max,
                    /* MAC src  */  g_source_mac_for_open_iface,
                    /* IP src   */  0,
                    /* IP dst   */  ip,
                    /* 0 = ARP request*/ 0);

            this->current_ip.Set(ip);

            /* Keep checking to see if main GUI thread wants this thread to end! */
            if ( this->is_waiting.Get_And_Then_Set_False() )
            {
                return 0;
            }

            this->Sleep(this->ms_btwn_arps); /* e.g 10 milliseconds */
        }
    }

    return true;
}

void *ArperThread::Entry()
{
    for (;;)
    {
        /* Continuously send ARP's to all specified network addresses */

        typedef SuperArray<NetworkAddress,const_max_qty_nets_to_arp>::const_iterator CI;

        for (CI it = this->nets.begin(); it != this->nets.end(); ++it)
        {
            if ( !DoOneNetwork(*it) )
            {
                /* The DoOneNetwork function regularly calls
                   Test_Is_Waiting_And_Reset, and it returns false if
                   Test_Is_Waiting_And_Reset returns true;
                */

                return 0; /* To simply end the thread */
            }
        }
    }

    return 0; /* Just to suppress compiler warning */
}
