#include "database.hpp"

#include <algorithm> /* find, sort */
#include <cassert>
#include <stdint.h>

#include "sniffer_custom_wx_event.hpp"  /* For sending updates to the main GUI dialogue box */

/* Returns an interator, and also a bool which is true when an alteration was made */
template<class Container, typename T>
std::pair<typename Container::iterator,bool> Sniffer_Database::find_in_container_or_add(Container &c, T const &val)
{
    typename Container::iterator const it = std::find( c.begin(), c.end(), val );

    if ( c.end() != it )  /* It was already in the container! */
        return std::pair<typename Container::iterator,bool>( it, false );

    try
    {
        wxCriticalSectionLocker tmp_object( this->cs );

        c.push_back(val);  /* This might throw! */

        return std::pair<typename Container::iterator,bool>(c.end() - 1, true);
    }
    catch (...)
    {
        return std::pair<typename Container::iterator,bool>(c.end(),false);  /* c.end() to indicate failure */
    }
}

void Sniffer_Database::SubmitData_For_One_Direction(uintMAC const mac, uintIP const ip, uintPort const port)
{
    On_Screen_Data_For_MAC osdfm;  /* osdfm.mac is initialised to zero within constructor */

    typedef Sniffer_Database::ContainerType_MAC::iterator ItMac;
    typedef MAC_Data::ContainerType_IP_Data::iterator ItIP;
    typedef IP_Data::ContainerType_Port_Data::iterator ItPort;

    if ( mac )
    {
        std::pair<ItMac,bool> it_mac = this->find_in_container_or_add(this->mac_data_container, mac);

        if ( this->mac_data_container.end() != it_mac.first ) /* if mac iterator is valid */
        {
            if ( it_mac.second ) /* if a new MAC was added to Database */
            {
                osdfm.mac = mac;  /* This is checked for 0 at end of function */
            }

            if ( ip )
            {
                std::pair<ItIP,bool> it_ip = this->find_in_container_or_add(it_mac.first->ip_data_container, ip);

                if ( it_mac.first->ip_data_container.end() != it_ip.first ) /* if ip iterator is valid */
                {
                    if ( it_ip.second )  /* if a new IP addition was made to container */
                    {
                        osdfm.mac = mac;  /* Will have been zero if MAC is not new */

                        /* LOCK the Database here! */

                        wxCriticalSectionLocker tmp_object( this->cs );

                        if ( IP_Classifier::IsPublicIP(ip) )
                            ++it_mac.first->qty_public_ip;
                        else
                            ++it_mac.first->qty_private_ip;

                        std::sort(it_mac.first->ip_data_container.begin(),
                                  it_mac.first->ip_data_container.end());
                    }

                    if ( port )
                    {
                        std::pair<ItPort,bool> it_port = this->find_in_container_or_add(it_ip.first->port_data_container, port);

                        if ( it_ip.first->port_data_container.end() != it_port.first )  /* if port iterator is valid */
                        {
                            if ( it_port.second ) /* if a new Port addition was made to container */
                            {
                                if ( it_mac.first->on_screen_ports.size() < const_max_qty_ports_on_screen )
                                {
                                    std::pair<ItPort,bool> it_port_on_screen = this->find_in_container_or_add(it_mac.first->on_screen_ports, port);

                                    if ( it_mac.first->on_screen_ports.end() != it_port_on_screen.first )
                                    {
                                        if ( it_port_on_screen.second ) /* if is unique addition */
                                        {
                                            osdfm.mac = mac; /* to indicate that a message must be sent to GUI */

                                            /* Nothing else to do here! */
                                        }
                                    }
                                }
                            }
                        }

                    } /* End of if (port) */

                } /* End of if (ip iterator is valid) */

            } /* End of if (ip) */

            if ( osdfm.mac ) /* if message will be sent to GUI thread, must populate IP's and Port's! */
            {
                osdfm.qty_private_ip = it_mac.first->qty_private_ip;
                osdfm.qty_public_ip = it_mac.first->qty_public_ip;

                /* OK, now populate the OSDFM object with ip's */

                ItIP p_ipdata  = it_mac.first->ip_data_container.begin();

                for ( IntType_Qty_IPs_On_Screen i = 0; i != const_max_qty_ips_on_screen; ++i, ++p_ipdata )
                {
                    if (it_mac.first->ip_data_container.end() == p_ipdata)
                        break;

                    try { osdfm.container_of_ips.push_back(p_ipdata->num); } catch (...) {}
                }

                ItPort p_portdata = it_mac.first->on_screen_ports.begin();

                for ( IntType_Qty_Ports_On_Screen i = 0; i != const_max_qty_ports_on_screen; ++i, ++p_portdata)
                {
                    if (it_mac.first->on_screen_ports.end() == p_portdata)
                        break;

                    try { osdfm.container_of_ports.push_back(p_portdata->num); } catch (...) {}
                }
            }

        }  /* End of if (iterator is valid) */

    }  /* End of if (mac) */


    if ( osdfm.mac )
    {
        EventClass_Sniffer event(g_event_type_sniffer,0);

        event.osdfm = osdfm;

        g_p_dlgmain->GetEventHandler()->AddPendingEvent(event);
            /* The code for this is in GUI_Dialog_Main_Populate_ListCtrl.cpp */
    }
}

void Sniffer_Database::SubmitFrameAnalysis(FrameAnalysis const &fa)
{
    /* All values that aren't "interesting" have already been
       set to zero by the FrameAnalyser class! */

    SubmitData_For_One_Direction(fa.mac_src,fa.ip_src,fa.port_src);
    SubmitData_For_One_Direction(fa.mac_dst,fa.ip_dst,fa.port_dst);
}

Sniffer_Database *g_p_sdb = 0;

#if 0
                    if ( port )
                    {
                        typedef IP_Data::ContainerType_Port_Data::iterator ItPort;

                        std::pair<ItPort,bool> z = this->find_in_container_or_add(y.first->port_data_container, port);

                        if ( y.first->port_data_container.end() != z.first )  /* if iterator is valid */
                        {
                            if ( z.second ) /* if a new addition was made */
                            {
                                /* Test for const_max_qty_ports_on_screen */
                                osdfm.mac = mac;

                                /* The IP container is either empty or already filled */

                                try { osdfm.container_of_ports.push_back(port); } catch (...) {}
                            }

                            /* And we're DONE! */
                        }
                    }
#endif
