#include "frame_analysis.hpp"

#include "reusable/lib_rawsock/frame.h"     /* Frame offsets, GetTransPtrFromIP4Ptr */
#include "reusable/lib_rawsock/checksum.h"  /* IsValidIP4 */
#include "reusable/lib_rawsock/send.h"      /* SendARPReply -- for phantom host */

#define ErrPrint(...) /* Nothing */

void FrameAnalysis::Process_Eth(uint8_t *const p_eth, uint8_t const *const pend)
{
    if (pend-p_eth < 14)  /* 6 + 6 + 2 = 14, discard frame if not long enough */
        return;

    this->mac_dst = Get48(p_eth+OFFSET_ETHER_DST);
    this->mac_src = Get48(p_eth+OFFSET_ETHER_SRC);

    switch ( Get16(p_eth+OFFSET_ETHER_PROTO) )
    {
    case 0x0800:

       this->Process_IP(p_eth+OFFSET_ETHER_over,pend);
       break;

    case 0x0806:

       this->Process_ARP(p_eth+OFFSET_ETHER_over,pend);
       break;
    }
}

void FrameAnalysis::Process_ARP(uint8_t *p_arp, uint8_t const *const pend)
{
    ErrPrint("Received ARP\n");

    /* Make sure the length is OK */

    /* Check for different kinds of ARP packets (e.g. IPX) */

    /* Log the source IP if it's not 0.0.0.0 */

    this->ip_src = Get32(p_arp + OFFSET_ARP_SRC_IP);

    switch(Get16(p_arp+OFFSET_ARP_OPCODE))
    {
    case 1:  /* ARP Request */

        /* A gratuitous ARP request is an AddressResolutionProtocol
           request packet where the source and destination IP are
           both set to the IP of the machine issuing the packet and
           the destination MAC is the broadcast address ff:ff:ff:ff:ff:ff */

        /* Now to respond to ARP requests for the phantom host */

/* THE RESPOND TO ARP REQUEST WAS ALSO SOMWHERE ELSE IN THE CODE
   BUT IT'S BEEN COMMENTED OUT */

/*
        if (Get32(p_arp+OFFSET_ARP_DST_IP) == g_src_ip)
        {
            ErrPrint("Responding to ARP for phantom\n");
            SendARP(g_s,Get48(p_arp+OFFSET_ARP_SRC_MAC),g_src_mac,g_src_ip,Get32(p_arp+OFFSET_ARP_SRC_IP),1);
        }
        break;
*/


    case 2:  /* ARP Reply */

        /* Some machines send ARP replies to their own MAC address! */
        if ( Get32(p_arp + OFFSET_ARP_SRC_MAC) != Get32(p_arp + OFFSET_ARP_DST_MAC) )
        {
            if ( Get32(p_arp + OFFSET_ARP_SRC_IP) != Get32(p_arp + OFFSET_ARP_DST_IP) )
            {
                this->ip_dst = Get32(p_arp + OFFSET_ARP_DST_IP);
            }
        }

        break;

    default:

        ErrPrint("ARP Opcode == %u\n",(unsigned)Get16(p_arp+OFFSET_ARP_OPCODE));
    }
}

void FrameAnalysis::Process_IP(uint8_t *p_ip, uint8_t const *const pend)
{
    if ( !IsValidIP4(p_ip,pend) )
    {
        ErrPrint("Invalid IPv4 Packet\n");
        return;
    }

    this->ip_src = Get32(p_ip + OFFSET_IP_SRC);
    this->ip_dst = Get32(p_ip + OFFSET_IP_DST);

    switch ( p_ip[OFFSET_IP_PROTO] ) /* This is OK coz it's 8-Bit */
    {
    case 0x06: /* TCP */

        this->Process_IP4_TCP(p_ip,pend);
        break;

    case 0x11: /* UDP */

        this->Process_IP4_UDP(p_ip,pend);
        break;
    }
}

void FrameAnalysis::Process_IP4_TCP(uint8_t *const p_ip, uint8_t const *const pend)
{
    if ( !IsValidIP4_TCP(p_ip,pend) )
    {
        ErrPrint("Invalid TCP Segment\n");
        return;
    }

    /* Get a pointer to the start of the segment */
    uint8_t const *p_trp = p_ip + GetIP4HeaderLen(p_ip);

    this->port_src = Get16(p_trp+OFFSET_TCP_SRC);
    this->port_dst = Get16(p_trp+OFFSET_TCP_DST);
}

void FrameAnalysis::Process_IP4_UDP(uint8_t *const p_ip, uint8_t const *const pend)
{
    if ( !IsValidIP4_UDP(p_ip,pend) )
    {
        ErrPrint("Invalid UDP Segment\n");
        return;
    }

    /* Get a pointer to the start of the segment */
    uint8_t const *p_trp = p_ip + GetIP4HeaderLen(p_ip);

    this->port_src = Get16(p_trp+OFFSET_UDP_SRC);
    this->port_dst = Get16(p_trp+OFFSET_UDP_DST);
}
