#ifndef HPP__FRAME_ANALYSIS
#define HPP__FRAME_ANALYSIS

#include <stdint.h> /* uint8_t */
#include "net_types_basic.hpp"  /* uintMAC, uintIP, uintPort */

class FrameAnalysis {
public:

    uintMAC mac_src, mac_dst;
    uintIP ip_src, ip_dst;
    uintPort port_src, port_dst;

private:

    static bool Is_MAC_Interesting(uintMAC const x)
    {
        if (x != 0 && x != const_mac_max)
            return true;

        return false;
    }

    static bool Is_IP_Interesting(uintIP const x)
    {
        if (x != 0 && x != const_ip_max)
            return true;

        return false;
    }

    static bool Is_Port_Interesting(uintPort const x)
    {
        if ( (x <= 1024u) && (x != 0) )
            return true;

        return false;
    }

    void Process_Eth(uint8_t *p_eth, uint8_t const *pend);
    void Process_ARP(uint8_t *p_arp, uint8_t const *pend);
    void Process_IP(uint8_t *p_ip, uint8_t const *pend);
    void Process_IP4_TCP(uint8_t *p_ip, uint8_t const *pend);
    void Process_IP4_UDP(uint8_t *p_ip, uint8_t const *pend);

public:

    FrameAnalysis(uint8_t *p, uint8_t const *pend)
      : mac_src(),mac_dst(),ip_src(),ip_dst(),port_src(),port_dst()
    {
        Process_Eth(p,pend);

        if ( !Is_MAC_Interesting(mac_src) ) mac_src = 0;
        if ( !Is_MAC_Interesting(mac_dst) ) mac_dst = 0;
        if ( !Is_IP_Interesting(ip_src) ) ip_src = 0;
        if ( !Is_IP_Interesting(ip_dst) ) ip_dst = 0;
        if ( !Is_Port_Interesting(port_src) ) port_src = 0;
        if ( !Is_Port_Interesting(port_dst) ) port_dst = 0;
    }
};

#endif
