#include "ip_classify.hpp"

#include <cassert>

#define IP_LIT(a,b,c,d)                  \
        (                            \
           (uintIP)(a) << 24  \
         | (uintIP)(b) << 16  \
         | (uintIP)(c) << 8   \
         | (uintIP)(d)        \
        )

namespace IP_Classifier {

SuperArray<NetworkAddress,const_max_qty_prv_nets> &Get_Prv_Nets_Container()
{
    static bool first_time = true;
    static SuperArray<NetworkAddress,const_max_qty_prv_nets> prv_nets;

    if (first_time)
    {
        first_time = false;

        try
        {
            prv_nets.push_back( NetworkAddress( IP_LIT( 10,  0,  0,  0),  8) );
            prv_nets.push_back( NetworkAddress( IP_LIT(169,254,  0,  0), 16) );
            prv_nets.push_back( NetworkAddress( IP_LIT(172, 16,  0,  0), 12) );
            prv_nets.push_back( NetworkAddress( IP_LIT(192,168,  0,  0), 16) );
        }
        catch (...) {}
    }

    return prv_nets;
}

bool IsPartOfNetwork(uintIP const ip,NetworkAddress const na)
{
    /* Network Address must be "clean" i.e. no host bits set */

    assert(na.qty_net_bits <= 30);

    uintIP const netmask = (const_ip_max << (32 - na.qty_net_bits)) & const_ip_max;
                   /* Need to AND it in case we have big ints on some systems */

    return (ip & netmask) == na.addr;
}

void DiscardAllPrivateNetworks(void)
{
    Get_Prv_Nets_Container().clear();
}

void AddPrvNet(NetworkAddress const na)
{
    /* The network address must be "clean", i.e. no host bits set! */

    typedef SuperArray<NetworkAddress,const_max_qty_prv_nets>::iterator It;

    for (It it = Get_Prv_Nets_Container().begin(); it != Get_Prv_Nets_Container().end(); ++it)
    {
        if ( IsPartOfNetwork(na.addr,*it) )
        {
            /* We already have this network
               in the list, no need to
               do anything.          */
            return;
        }
    }

    try { Get_Prv_Nets_Container().push_back(na); } catch (...) {}

    /* If the list is already full, nothing happens */
}


bool IsPublicIP(uintIP const ip)
{
    typedef SuperArray<NetworkAddress,const_max_qty_prv_nets>::const_iterator CIt;

    for (CIt cit = Get_Prv_Nets_Container().begin(); cit != Get_Prv_Nets_Container().end(); ++cit)
    {
        if ( IsPartOfNetwork(ip,*cit) )
            return false;
    }

    return true;
}

#if 0
IP_Qty QuantityIPs(MAC_Data const *p_macdata,bool const is_public)
{
    IP_Qty quantity = 0;

    for (MAC_Data::Container_Type_IP::const_iterator p  = p_macdata->ip_data_container.begin();
                                                     p != p_macdata->ip_data_container.end();
                                                     ++p)
    {
        if ( IsPublicIP(p->num) == is_public )
        {
            ++quantity;
        }
    }

    return quantity;
}
#endif

} /* Close IP_Classifier Namespace */
