#ifndef HPP__NET_TYPES_CONTAINERS
#define HPP__NET_TYPES_CONTAINERS

#include <stdint.h>

#include "master_specs_for_dynamo_program.hpp" /* const_ip_max_qty, const_mac_max_qty */

#include "net_types_basic.hpp"  /* uintMAC, uintIP, uintPort */

#include "reusable/superarray.hpp"

#include "ip_classify.hpp"  /* IP_Classifier::IsPublicIP */


/* ---------------- First here come the types ---------------- */

struct Port_Data {

public:

    uintPort num;

    Port_Data(uintPort const arg) : num(arg) {}

    bool operator==(uintPort const arg) const  /* used by std::find */
    {
        return this->num == arg;
    }

#if 0
private:

    /* ------ Prevent copy-construction and assignment------ */
    Port_Data(Port_Data const &);

    Port_Data &operator=(Port_Data const &);
    /* ----------------------------------------------------- */
#endif
};

struct IP_Data {

public:

    uintIP num;

    IP_Data(uintIP const arg) : num(arg) {}

    bool operator==(uintIP const arg) const  /* used by std::find */
    {
        return this->num == arg;
    }

    typedef SuperArray<Port_Data,const_port_max_qty> ContainerType_Port_Data;

    ContainerType_Port_Data port_data_container;

    bool operator<(IP_Data const &rhs) const  /* used by std::sort */
    {
        /*
        if (a == g_inet_dst_ip) return 1;
        if (b == g_inet_dst_ip) return 0;
        */

        bool const a_pub = IP_Classifier::IsPublicIP(this->num),
                   b_pub = IP_Classifier::IsPublicIP(rhs.num);

        if (!a_pub && b_pub)
            return true;

        if (a_pub && !b_pub)
            return false;

        if (this->num < rhs.num)
            return true;

        return false;
    }

#if 0
private:

    /* ------ Prevent copy-construction and assignment------ */
    IP_Data(IP_Data const &);

    IP_Data &operator=(IP_Data const &);
    /* ----------------------------------------------------- */
#endif
};

struct MAC_Data {

public:

    uintMAC num;

    IntType_Qty_IP qty_private_ip, qty_public_ip;

    SuperArray<Port_Data,const_max_qty_ports_on_screen> on_screen_ports;

    MAC_Data(uintMAC const arg) : num(arg), qty_private_ip(0), qty_public_ip(0) {}

    typedef SuperArray<IP_Data,const_ip_max_qty> ContainerType_IP_Data;

    ContainerType_IP_Data ip_data_container;

    bool operator==(uintMAC const arg) const  /* used by std::find */
    {
        return this->num == arg;
    }

    bool operator<(MAC_Data const &rhs) const /* used by std::sort */
    {
        return this->num < rhs.num;
    }

#if 0
private:

    /* ------ Prevent copy-construction and assignment------ */
    MAC_Data(MAC_Data const &rhs);

    MAC_Data &operator=(MAC_Data const &);
    /* ----------------------------------------------------*/
#endif
};

#endif

