#ifndef H__ETHERSOCK
#define H__ETHERSOCK

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>


#ifndef VJC_LIB_RAW_SOCK_UNDERLYING_API

#   error "Must specify underlying API by defining VJC_LIB_RAW_SOCK_UNDERLYING_API"

#elif VJC_LIB_RAW_SOCK_UNDERLYING_API == 0 /* This is the dummy */

    typedef struct RawEtherSockHandle {

        uint_fast8_t s;

    } RawEtherSockHandle;

    static RawEtherSockHandle const g_sample_bad_ethersock = {0};

    #define SAMPLE_BAD_ETHERSOCK ((RawEtherSockHandle)g_sample_bad_ethersock)

    #define IS_BAD_ETHERSOCK(h) (0 == (h).s)

    typedef struct FrameInfo {
        uint8_t buf[65535];
        uint_fast16_t len;
    } FrameInfo;

#elif VJC_LIB_RAW_SOCK_UNDERLYING_API == 1 /* Native Linux API */

    typedef struct RawEtherSockHandle {

        int s;

    } RawEtherSockHandle;

    static RawEtherSockHandle const g_sample_bad_ethersock = {-1};

    #define SAMPLE_BAD_ETHERSOCK ((RawEtherSockHandle)g_sample_bad_ethersock)

    #define IS_BAD_ETHERSOCK(h) (-1 == (h).s)

    typedef struct FrameInfo {
        uint8_t buf[65535];
        uint_fast16_t len;
    } FrameInfo;

#elif VJC_LIB_RAW_SOCK_UNDERLYING_API == 2 /* pcap */

#   include "rawsock_pcap.h"

#else

#    error "Unrecognised value for VJC_LIB_RAW_SOCK_UNDERLYING_API"

#endif

RawEtherSockHandle OpenRawEtherSock(char const *nic, int const promisc);

int RecvRawEtherFrame(RawEtherSockHandle,FrameInfo *);

void SendRawEtherFrame(RawEtherSockHandle,uint8_t const *,uint_fast16_t);

void CloseRawEtherSock(RawEtherSockHandle);

char const *GetListOfInterfaces(uint_fast8_t i);

uint_fast64_t GetInterfaceMAC(char const *nic);

#ifdef __cplusplus
	} /* To close the extern "C" */
#endif

#endif
