#ifdef __cplusplus
#error You must use a C compiler to compile this file (not a C++ compiler)
#endif

#include "send.h"

#include <stdint.h>
#include "rawsock.h" /* RawEtherSockHandle, SendEtherFrame */
#include "checksum.h"  /* MakeValid_IP4_Transport */
#include "frame.h"     /* Frame Offsets, Get16, Set16, etc. */
#include "abbrev.h"    /* PEND */

void SendARP(RawEtherSockHandle const h,
             uint_fast64_t const mac_dst,
             uint_fast64_t const mac_src,
             uint_fast32_t const ip_src,
             uint_fast32_t const ip_dst,
             uint_fast8_t const is_reply)
{
    static uint8_t frame[] =
    {
      /* Offset 0 : Beginning of Ethernet frame */

      0,0,0,0,0,0,  /* Dest MAC == MAC of possible router */
      0,0,0,0,0,0,  /* Src  MAC == Our MAC */
        0x08,0x06,  /* eth.type == IP */

      /* Offset 14 : Beginning of ARP packet */

      0x00,0x01,    /* Hardware type == Ethernet */
      0x08,0x00,    /* Protocol type == IP */
      0x06,         /* Hardware size */
      0x04,         /* Protocol size */
      0x00,0x01,    /* ARP type = request */
      0,0,0,0,0,0,  /* Src MAC */
      0,0,0,0,      /* Src IP */
      0,0,0,0,0,0,  /* Dst MAC */
      0,0,0,0       /* Dst IP */
    };

    Set48(frame+OFFSET_ETHER_DST,mac_dst);
    Set48(frame+OFFSET_ETHER_over+OFFSET_ARP_DST_MAC,mac_dst);

    Set48(frame+OFFSET_ETHER_SRC,mac_src);
    Set48(frame+OFFSET_ETHER_over+OFFSET_ARP_SRC_MAC,mac_src);

    Set32(frame+OFFSET_ETHER_over+OFFSET_ARP_SRC_IP,ip_src);
    Set32(frame+OFFSET_ETHER_over+OFFSET_ARP_DST_IP,ip_dst);

    if (is_reply)
        frame[OFFSET_ETHER_over+OFFSET_ARP_OPCODE+1] = 2;

    SendRawEtherFrame(h,frame,sizeof frame);
}

void SendDNSLookup(RawEtherSockHandle const h,
                   uint_fast64_t const mac_dst,
                   uint_fast64_t const mac_src,
                   uint_fast32_t const ip_src,
                   uint_fast32_t const ip_dst)
{
    static uint8_t frame[] =
    {
      /* Offset 0 : Beginning of Ethernet frame */

      0,0,0,0,0,0,  /* Dest MAC == MAC of possible router */
      0,0,0,0,0,0,  /* Src  MAC == Our MAC */
        0x08,0x00,  /* eth.type == IP */

      /* Offset 14 : Beginning of IP packet */

      0x45,        /* IP Version and header length */
      0x00,        /* Type of Service */
      0,0,         /* Total Length */
      0x00,0x00,   /* Identification */
      0x40,0x00,   /* Flags and Fragment Offset */
      0xFF,        /* Time To Live */
      0x11,        /* Transport Layer Protocol == UDP */

      0,0,         /* IP Header Checksum */

      0,0,0,0,     /* Src IP == Our IP Address */
      0,0,0,0,     /* Dest IP == DNS server */

      /* Offset 34 : Beginning of UDP segment */

      0x80,0x01,   /* Source port */
      0x00,0x35,   /* Destination port */
      0,0,         /* Length of UDP segment (header + data) */
      0,0,         /* Checksum */

      /* Offset 42 : Beginning of DNS datagram */

      0x22,0xf4,  /* Transaction ID */
      0x01,0x00,  /* Flags */
      0x00,0x01,  /* Questions */
      0x00,0x00,  /* Answer RR's */
      0x00,0x00,  /* Athority RR's */
      0x00,0x00,  /* Additional RR's */

      3,'w','w','w',6,'g','o','o','g','l','e',3,'c','o','m',0,

      0x00,0x01,   /* Type A (Host Address) */
      0x00,0x01    /* Class IN */
    };

    Set48(frame+OFFSET_ETHER_DST,mac_dst);
    Set48(frame+OFFSET_ETHER_SRC,mac_src);
    Set32(frame+OFFSET_ETHER_over+OFFSET_IP_SRC,ip_src);
    Set32(frame+OFFSET_ETHER_over+OFFSET_IP_DST,ip_dst);

    MakeValid_IP4_Transport(frame+OFFSET_ETHER_over,PEND(frame));

    SendRawEtherFrame(h,frame,sizeof frame);
}

void SendTCPSyn(RawEtherSockHandle const h,
                   uint_fast64_t const mac_dst,
                   uint_fast64_t const mac_src,
                   uint_fast32_t const ip_src,
                   uint_fast32_t const ip_dst,
                   uint_fast16_t const port_src,
                   uint_fast16_t const port_dst)
{
    static uint8_t frame[] =
    {
      /* Offset 0 : Beginning of Ethernet frame */

      0,0,0,0,0,0,  /* Dest MAC == MAC of possible router */
      0,0,0,0,0,0,  /* Src  MAC == Our MAC */
        0x08,0x00,  /* eth.type == IP */

      /* Offset 14 : Beginning of IP packet */

      0x45,        /* IP Version and header length */
      0x00,        /* Type of Service */
      0,0,         /* Total Length */
      0xd0,0x6c,   /* Identification */ /* ??????????????????????????????????? */
      0x40,0x00,   /* Flags and Fragment Offset */
      0xFF,        /* Time To Live */
      0x06,        /* Transport Layer Protocol == TCP */

      0,0,         /* IP Header Checksum */

      0,0,0,0,     /* Src IP == Our IP Address */
      0,0,0,0,     /* Dest IP == DNS server */

      /* Offset 34 : Beginning of TCP segment */

      0,0,                  /* Source port */
      0,0,                  /* Destination port */
      0x00,0x00,0x00,0x00,   /* Sequence Number */
      0x00,0x00,0x00,0x00,   /* Acknowledgement Number */
      0xac,                 /* Header length and reserved */
      0x02,                 /* Flags such as Syn, Fin, Ack */
      0x16,0xd0,            /* Windows size == 16384 */
      0,0,                  /* Checksum */
      0x00,0x00,            /* Urgent Pointer */

      /* Here comes 20 bytes of funky options: */

      0x02,0x04,0x05,0xb4,0x04,0x02,0x08,0x0a,0x00,0x1a,
      0xc0,0x4a,0x00,0x00,0x00,0x00,0x01,0x03,0x03,0x06

    };

    Set48(frame+OFFSET_ETHER_DST,mac_dst);
    Set48(frame+OFFSET_ETHER_SRC,mac_src);
    Set32(frame+OFFSET_ETHER_over+OFFSET_IP_SRC,ip_src);
    Set32(frame+OFFSET_ETHER_over+OFFSET_IP_DST,ip_dst);
    Set16(frame+OFFSET_ETHER_over+OFFSET_IP_over+OFFSET_TCP_SRC,port_src);
    Set16(frame+OFFSET_ETHER_over+OFFSET_IP_over+OFFSET_TCP_DST,port_dst);

    MakeValid_IP4_Transport(frame+OFFSET_ETHER_over,PEND(frame));

    SendRawEtherFrame(h,frame,sizeof frame);
}
