#ifndef HPP__SUPERARRAY
#define HPP__SUPERARRAY

/* --------------------------------------------------- */
#if 0

/*
Don't change this to vector because the program depends
on begin() never changing. If you use vector, there's a
chance that the vector will reallocate memory and change
the begin() address, which would invalidate the MAC_Data
pointer held by the "Show Detailed MAC Info" Dialog Box.
*/

#include <vector>

template<class T, unsigned len>
class SuperArray : public std::vector<T> {

public:

    template<class X>
    void push_back(X const &val)
    {
        if ( this->size() == len )
        {
            /* Wups we're full! */
            throw -1;
        }

        std::vector<T>::push_back(val);
    }

};
#endif
/* --------------------------------------------------- */


#if 1

#include <cassert>
#include <cstddef>  /* size_t */
#include <new>  /* for "placement new" */

template< typename T, std::size_t len >
class SuperArray {
public:

    /* ---------- First here comes the types ---------- */
    typedef std::size_t size_type;

    typedef T value_type;

    typedef T &reference;
    typedef T const &const_reference;

    typedef T *iterator;
    typedef T const *const_iterator;
    /* ------------------------------------------------ */

private:

    char unsigned raw_mem[ sizeof( T[len] ) ];

    T *const pbegin;

    T *pend;

    T const *const pend_of_allocated_space;

public:

#   define init_list_for_SuperArray                                \
                                                                   \
         pbegin( static_cast<T*>( static_cast<void*>(raw_mem) ) ), \
         pend( pbegin ),                                           \
         pend_of_allocated_space( pbegin + len )

    SuperArray() : init_list_for_SuperArray
    {

    }

    SuperArray(SuperArray const &original) : init_list_for_SuperArray
    {
        for (typename SuperArray::const_iterator ci = original.begin(); ci != original.end(); ++ci )
            this->push_back(*ci);
    }

#   undef init_list_for_SuperArray

    SuperArray &operator=(SuperArray const &rhs)
    {
        this->clear();

        for (typename SuperArray::const_iterator ci = rhs.begin(); ci != rhs.end(); ++ci )
            this->push_back(*ci);

        return *this;
    }

public:

    bool empty() const { return pend == pbegin; }

    //size_type capacity() const { return len; }
    //size_type max_size() const { return len; }

    size_type size() const { return pend - pbegin; }

    reference operator[](size_t const i)
    {
        assert( this->size() > i );

        return pbegin[i];
    }

    const_reference operator[](size_t const i) const
    {
        assert( this->size() > i );

        return pbegin[i];
    }

    iterator begin()
    {
        /* It's OK for container to be empty */
        return pbegin;
    }

    const_iterator begin() const
    {
        /* It's OK for container to be empty */
        return pbegin;
    }

    iterator end()
    {
        /* It's OK for container to be empty */
        return pend;
    }

    const_iterator end() const
    {
        /* It's OK for container to be empty */
        return pend;
    }

    reference front() { assert( !this->empty() ); return *pbegin; }
    const_reference front() const { assert( !this->empty() ); return *pbegin; }

    reference back() { assert( !this->empty() ); return pend[-1]; }
    const_reference back() const { assert( !this->empty() ); return pend[-1]; }

    template<class X>
    void push_back(X const &val)
    {
        if ( pend_of_allocated_space == pend )
        {
            /* Wups we're full! */
            throw -1;
        }

        /* Let the following throw if it wants to */
        ::new(pend) T(val);

        ++pend;  /* This doesn't happens if the placement new fails */
    }

    void pop_back()
    {
        assert( !this->empty() );

        this->back().~T();

        --this->pend;
    }

    void clear()
    {
        while ( !this->empty() )
            this->pop_back();
    }
};

#endif

#endif
